package cn.pluss.platform.mapper;

import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Mapper;
import cn.pluss.platform.entity.DeviceMerchantBuy;
import cn.pluss.platform.vo.DeviceMerchantBuyVO;
import org.apache.ibatis.annotations.Param;

@Mapper
public interface DeviceMerchantBuyMapper extends BaseMapper<DeviceMerchantBuy> {

    DeviceMerchantBuy queryDeviceMerchantBuy(DeviceMerchantBuy deviceMerchantBuy);

    List<DeviceMerchantBuy> queryDeviceMerchantBuyList(DeviceMerchantBuy deviceMerchantBuy);

    void saveDeviceMerchantBuy(DeviceMerchantBuy deviceMerchantBuy);

    void updateDeviceMerchantBuy(DeviceMerchantBuy deviceMerchantBuy);

    void deleteDeviceMerchantBuy(DeviceMerchantBuy deviceMerchantBuy);

    List<DeviceMerchantBuy> queryDeviceMerchantBuyPage(Map map);

    /**
     * 查询分页数据
     * @param page      分页信息
     * @param wrapper   查询条件
     * @return
     */
    Page<DeviceMerchantBuy> selectDeviceMerchantBuyPage(Page<DeviceMerchantBuy> page, @Param(Constants.WRAPPER) QueryWrapper<DeviceMerchantBuy> wrapper);

    Integer queryDeviceMerchantBuyPageCount(Map map);

    void saveDeviceMerchantBuyBatch(List<DeviceMerchantBuy> deviceMerchantBuyList);

    /**
     * deviceOrderShowPage:(订单回显). <br/>
     *
     * @param map
     * @return
     *
     * @author Administrator
     * @since JDK 1.8
     */
    List<DeviceMerchantBuyVO> deviceOrderShowPage(Map<String, Object> map);

    /**
     * deviceOrderShowPageCount:(订单回显条数统计). <br/>
     *
     * @param map
     * @return
     *
     * @author Administrator
     * @since JDK 1.8
     */
    Integer deviceOrderShowPageCount(Map<String, Object> map);

}

